@@ -112,7 +112,7 @@ test_that("contour plot tests", {
                                     kd = 1:3, ky = 1:3))
 
   # formula
-  expect_invisible(ovb_contour_plot(peacefactor ~ directlyharmed + age + farmer_dar +
+  expect_invisible(ovb_contour_plot(formula = peacefactor ~ directlyharmed + age + farmer_dar +
                                       herder_dar + pastvoted + hhsize_darfur + female + village, data = darfur,
                                     treatment = "directlyharmed",
                                     benchmark_covariates = "female",
